/*
   Arduino LOG detector V 1.5
   LT5534 50-3000MHz on 3,3 Ucc pin
   Input Power Range: –60dBm to 0dBm
   (c) OK1DXD 2016,2017
*/

//EXTERNAL ADC reference - put a measured value on 3.3V pin here:
float refVolts = 3.26; // [Volts]
//float refVolts = 3.359; 
//float refVolts = 4.71;

#include <LiquidCrystal.h>
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

int pin = 2; // input analog pin
int logDetOut   = 0;
float voltage   = 0;
int adc_key_in  = 0;
float powerdB   = 0.00;   // power in dBm
float pmWatts   = 0.00;  // Calculated power in mWatts

//calibration tables dBm, miliVolts
int maxnumber = 10; //number of calibration table points
int frq_index = 1 ; // set cursor on the middle
int db[] = { -65, -60, -50, -40, -30, -20, -15, -10, -5, 0};
int frq[] = {10, 25, 50, 250, 500, 1000, 2000, 3000}; //frequency bands used [MHz]

//calibration table for 3,3V LOG det Ucc
int mv[8][10] = {
  {251, 382, 818, 1172, 1548, 1957, 2179, 2396, 2523, 2585}, //    10MHz
  {267, 407, 748, 1163, 1590, 2026, 2259, 2450, 2552, 2604}, //    25MHz
  {248, 366, 748, 1163, 1590, 2026, 2259, 2450, 2546, 2593}, //    50MHz
  {235, 337, 713, 1137, 1564, 1991, 2211, 2412, 2514, 2564}, //   250MHz
  {239, 340, 723, 1140, 1551, 1969, 2186, 2380, 2472, 2502}, //   500MHz
  {248, 360, 742, 1128, 1523, 1915, 2125, 2297, 2351, 2384}, //  1000MHz
  {245, 360, 739, 1096, 1456, 1810, 2001, 2147, 2189, 2199}, //  2000MHz
  {245, 360, 739, 1096, 1456, 1810, 2001, 2147, 2189, 2199}  //  3000MHz
  };
int inmV = 600; // input value for testing purpose only
int i = 0;

// define a values used by the buttons
#define btnRIGHT  0
#define btnUP     1
#define btnDOWN   2
#define btnLEFT   3
#define btnSELECT 4
#define btnNONE   5

int lcd_key     = 0;

void setup() {

  // ADC reference to use (DEFAULT, INTERNAL, INTERNAL1V1, INTERNAL2V56, or EXTERNAL).
  // Very important note – when using an external voltage reference, 
  // you must set the analogue
  // reference to EXTERNAL before using ANY (!) analogRead
  
  analogReference(EXTERNAL);   // use AREF for reference voltage
  logDetOut = analogRead(pin); //at least one Analog read needed

  //welcome screen
  lcd.begin(16, 2);
  lcd.clear();
  lcd.print("LT5534 DET V1.5");
  lcd.setCursor(0, 1);
  lcd.print(" -60/0dBm MAX");
  delay (2000);
  lcd.clear();
}
// read the buttons function
int read_LCD_buttons()
{
  adc_key_in = analogRead(0);      
 // read the value from the ADC add approx 50 to those values and check to see if we are close
  if (adc_key_in > 1000) return btnNONE;
  /*
    // For Internal 5V ref use this threshold
     if (adc_key_in < 50)   return btnRIGHT;
    if (adc_key_in < 150)  return btnUP;
    if (adc_key_in < 300)  return btnDOWN;
    if (adc_key_in < 456)  return btnLEFT;
    if (adc_key_in < 700)  return btnSELECT; //637
  */
  // For External 3.3V ref use this threshold
  if (adc_key_in < 50)   return btnRIGHT;
  if (adc_key_in < 190)  return btnUP;
  if (adc_key_in < 408)  return btnDOWN;
  if (adc_key_in < 620)  return btnLEFT;
  if (adc_key_in < 940)  return btnSELECT; //637

  return btnNONE;  // when all others fail, return this...
}

// getdBValue returns value from the CALIBRATION table on given frq_index
float getdBValue(int inputmV) {
  int xA;
  int xB;
  int dx; //delta
  int dy;
  uint8_t index;
  float k; //slope
  for (int i = 0; i < maxnumber; i++) // number of calibration points
  {

    if (mv[frq_index][0] >= inputmV) // input value is under the detector range
    {
      return db[0];
      break; // done
    }

    if (mv[frq_index][i] > inputmV) // find the higher point = index
    {
      xA = mv[frq_index][i - 1];
      xB = mv[frq_index][i];
      index = i;
      break; // done
    }
  }
  dx = xB - xA;
  dy = db[index] - db[index - 1];
  k = (float) dy / (float) dx;
  return k * (float) (inmV - xA) + db[index - 1];
}
void loop() {
    // until button SELECT is pressed: display the RAW Analog data + convert to mV
  while (lcd_key == btnSELECT)
//   while (1 == 1) //for calibration loop
  {
    logDetOut = 0;
    for (i = 0; i < 5; i++)  {
        logDetOut = (logDetOut + analogRead(pin)); //read the ADC and store the result in value
        delay (100);   
      }
    logDetOut = (logDetOut/5); // get AVERAGE value
    
    voltage = (logDetOut * refVolts) / 1023; // Convert the ADC result to Volts in vout
    inmV = (int)( voltage * 1000);
    lcd.clear();
    lcd.print("RAW IN: ");
    lcd.setCursor(8, 0);
    lcd.print("    ");
    lcd.setCursor(8, 0);
    lcd.print(logDetOut);
    lcd.setCursor(0, 1);
    lcd.print("mV  IN: ");
    lcd.setCursor(8, 1);
    lcd.print("    ");
    lcd.setCursor(8, 1);
    lcd.print(inmV);
    lcd_key = read_LCD_buttons();
    delay(100);
  }

  lcd_key = read_LCD_buttons();  // read the buttons
  switch (lcd_key)  // depending on which button was pushed, performs an action
  {
    case btnUP:
      {
        lcd.setCursor(13, 1);
        lcd.print("UP ");
        frq_index = frq_index + 1;
        delay(100);
        if  (frq_index > (maxnumber - 1))
        {
          frq_index = (maxnumber - 1) ; // then last point
        }
        //
        break;
      }

    case btnDOWN:
      {
        lcd.setCursor(13, 1);
        lcd.print("DWN");
        delay(100);
        frq_index = frq_index - 1;
        if  (frq_index < 0)
        {
          frq_index = 0; // MIN mezni hodnota
        }
        break;
      }

    case btnNONE:
      {
        lcd.setCursor(13, 1);
        lcd.print("   "); //Clear
        break;
      }
  }

  // this reeds the Analog Pin data
  
    logDetOut = 0;
    for (i = 0; i < 5; i++)  {
        logDetOut = (logDetOut + analogRead(pin)); //read the ADC and store the result in value
        delay (100);   
        }
        logDetOut = (logDetOut/5); // get AVERAGE value
 
  voltage = (logDetOut * refVolts) / 1023; // Convert RAW ADC result to Volts in vout
  inmV = (int)( voltage * 1000);

  //  test on overload first
    while ( inmV >  (mv[frq_index][maxnumber - 1] + 10)) {
    lcd.clear();
    lcd.print("*INPUT > ");
    lcd.print(db[maxnumber - 1]);
    lcd.print("dBm *");
    logDetOut = analogRead(pin); //read the ADC and store the result in value
    voltage = (logDetOut * refVolts) / 1023; // Convert the ADC result to miliVolts
    inmV = (int)( voltage * 1000);
    lcd.setCursor(0, 1);
    lcd.print("In: ");
    lcd.setCursor(8, 1);
    lcd.setCursor(4, 1);
    lcd.print(inmV);
    lcd.print("mV");
    delay(500);
    lcd.setCursor(0, 0);
    lcd.print("                ");     //blink
    delay(500);
  }
  
    //  test on low input
    while  ( inmV <  (mv[frq_index][0]-5)) {
      lcd.clear();
      lcd.print("*INPUT < ");
      lcd.print(db[0]);
      lcd.print("dBm*");
      logDetOut = analogRead(pin); //read the ADC and store the result in value
      voltage = (logDetOut * refVolts) / 1023; // Convert the ADC result to Volts in vout
      inmV = (int)( voltage * 1000);
      delay(500);
    } 

  //  dBm value from the calibration table
  powerdB = getdBValue(inmV);
  pmWatts = 1000 * (pow(10.0, (powerdB) / 10.00)); // convert dBm to microWatts
  lcd.setCursor(0, 0);
  lcd.print("                "); //clear ROW 1
  lcd.setCursor(0, 0);
  lcd.print("dBm: ");
  lcd.setCursor(11, 0);
  lcd.print(frq[frq_index]);
  lcd.setCursor(5, 0); 
  
  if (powerdB < 0 )
   lcd.setCursor(4, 0); 
   
//  lcd.setCursor(4, 0);
  lcd.print(powerdB, 1);
  lcd.setCursor(0, 1);
  lcd.print(" uW: ");
  lcd.setCursor(5, 1);
  lcd.print("       ");
  lcd.setCursor(5, 1);
  lcd.print(pmWatts, 3);
  delay(100);

}
